"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const filters = {
    isNil: (propertyPath, ref) => (data) => lodash_1.get(data, propertyPath) === null || lodash_1.get(data, propertyPath) === undefined,
    isNotNil: (propertyPath, ref) => (data) => lodash_1.get(data, propertyPath) !== null && lodash_1.get(data, propertyPath) !== undefined,
    equals: (propertyPath, ref) => (data) => lodash_1.get(data, propertyPath) === ref,
    notEquals: (propertyPath, ref) => (data) => lodash_1.get(data, propertyPath) !== ref,
    lowerThan: (propertyPath, ref) => (data) => lodash_1.get(data, propertyPath) < ref,
    lowerOrEqualTo: (propertyPath, ref) => (data) => lodash_1.get(data, propertyPath) <= ref,
    greaterThan: (propertyPath, ref) => (data) => lodash_1.get(data, propertyPath) < ref,
    greaterOrEqualTo: (propertyPath, ref) => (data) => lodash_1.get(data, propertyPath) <= ref,
    contains: (propertyPath, ref) => (data) => {
        const propValue = lodash_1.get(data, propertyPath);
        if (Array.isArray(propValue)) {
            return propValue.includes(ref);
        }
        else {
            return String(propValue).includes(String(ref));
        }
    },
    startsWith: (propertyPath, ref) => (data) => {
        const propValue = lodash_1.get(data, propertyPath);
        if (Array.isArray(propValue)) {
            return propValue.indexOf(ref) === 0;
        }
        else {
            return String(propValue).startsWith(String(ref));
        }
    },
    endsWith: (propertyPath, ref) => (data) => {
        const propValue = lodash_1.get(data, propertyPath);
        if (Array.isArray(propValue)) {
            return propValue.indexOf(ref) === propValue.length - 1;
        }
        else {
            return String(propValue).endsWith(String(ref));
        }
    },
    isIn: (propertyPath, possibleValues) => (data) => {
        if (!Array.isArray(possibleValues)) {
            throw new Error(`Reference value of an isIn operation must be an Array. Expected Array but found ${typeof (possibleValues)}`);
        }
        return possibleValues.includes(lodash_1.get(data, propertyPath));
    },
    isNotIn: (propertyPath, possibleValues) => (data) => {
        if (!Array.isArray(possibleValues)) {
            throw new Error(`Reference value of an isNotIn operation must be an Array. Expected Array but found ${typeof (possibleValues)}`);
        }
        return !possibleValues.includes(lodash_1.get(data, propertyPath));
    }
};
const isGroup = (value) => {
    return value && typeof (value) === 'object' && 'type' in value && 'operations' in value;
};
const isEntry = (value) => {
    return value && typeof (value) === 'object' && 'propertyPath' in value && 'operation' in value /* && 'refValue' in value */;
};
var WarixDataFilterGroupJoin;
(function (WarixDataFilterGroupJoin) {
    WarixDataFilterGroupJoin["AND"] = "and";
    WarixDataFilterGroupJoin["OR"] = "or";
})(WarixDataFilterGroupJoin = exports.WarixDataFilterGroupJoin || (exports.WarixDataFilterGroupJoin = {}));
exports.WarixDataFilters = {
    isNil: filters.isNil,
    nil: filters.isNil,
    isNotNil: filters.isNotNil,
    nnil: filters.isNotNil,
    equals: filters.equals,
    eq: filters.equals,
    notEquals: filters.notEquals,
    neq: filters.notEquals,
    lowerThan: filters.lowerThan,
    lt: filters.lowerThan,
    lowerOrEqualTo: filters.lowerOrEqualTo,
    lte: filters.lowerOrEqualTo,
    greaterThan: filters.greaterThan,
    gt: filters.greaterThan,
    greaterOrEqualTo: filters.greaterOrEqualTo,
    gte: filters.greaterOrEqualTo,
    contains: filters.contains,
    cc: filters.contains,
    startsWith: filters.startsWith,
    sw: filters.startsWith,
    endsWith: filters.endsWith,
    ew: filters.endsWith,
    isIn: filters.isIn,
    in: filters.isIn,
    isNotIn: filters.isNotIn,
    nin: filters.isNotIn
};
exports.AND = (...operations) => (dataOwner) => {
    for (const op of operations) {
        if (!op(dataOwner)) {
            return false;
        }
    }
    return true;
};
exports.OR = (...operations) => (dataOwner) => {
    for (const op of operations) {
        if (op(dataOwner)) {
            return true;
        }
    }
    return false;
};
function compileGroup(group, ...providers) {
    const fnFindOperation = (operationName) => {
        for (const pr of providers) {
            if (operationName in pr) {
                return pr[operationName];
            }
        }
        if (operationName in exports.WarixDataFilters) {
            return exports.WarixDataFilters[operationName];
        }
        throw new Error(`WarixDataFilterException. Unknown operation ${operationName}. Are you missing an operations provider?`);
    };
    const operations = [];
    for (const op of group.operations) {
        if (isEntry(op)) {
            operations.push(fnFindOperation(op.operation)(op.propertyPath, op.propertyPath));
        }
        else if (isGroup(op)) {
            operations.push(compileGroup(op));
        }
        else {
            throw new Error(`WarixDataFilterException. Definition ${op} is neither a group nor an entry`);
        }
    }
    return group.type === WarixDataFilterGroupJoin.AND ? exports.AND(...operations) : exports.OR(...operations);
}
exports.compileGroup = compileGroup;
function groupContainsProperty(group, propertyPath) {
    for (const op of group.operations) {
        if (isEntry(op)) {
            if (op.propertyPath === propertyPath) {
                return true;
            }
        }
        else if (isGroup(op)) {
            if (groupContainsProperty(op, propertyPath)) {
                return true;
            }
        }
    }
    return false;
}
exports.groupContainsProperty = groupContainsProperty;
function groupProperties(group) {
    const fields = [];
    for (const op of group.operations) {
        if (isEntry(op)) {
            if (!fields.includes(op.propertyPath)) {
                fields.push(op.propertyPath);
            }
        }
        else if (isGroup(op)) {
            fields.push(...groupProperties(op).filter(propertyPath => fields.includes(propertyPath) === false));
        }
    }
    return fields;
}
exports.groupProperties = groupProperties;
//# sourceMappingURL=warix.data-filter.js.map